"""Argparse Wrapper Class for automatic config loading"""

import argparse
import os
import sys
from pathlib import Path

from .config import NETDBConfig, NETDBConfigError


class DeprecateAction(argparse.Action):
    def __init__(self, *args, **kwargs):
        self.call_count = 0
        if 'help' in kwargs:
            kwargs['help'] = f'[DEPRECATED] {kwargs["help"]}'
        super().__init__(*args, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        if self.call_count == 0:
            sys.stderr.write(f'The option `{option_string}` is deprecated. Help:\n')
            sys.stderr.write(f'{self.help}\n')
            sys.exit(1)


class ArgumentParser(argparse.ArgumentParser):
    """Argument parser with default common arguments for NetDB-api cli
    tools. Includes default values."""

    def __init__(
        self,
        epilog='The Variables BASE_URL, VERSION and TOKEN can be loaded from config file,'
        'environment or from command line arguments.\n'
        'Variable precendence is as followed (from greatest to least,'
        'means the first listed variable overwrites all other variables):\n'
        '  1. cli-arguments\n'
        '  2. environment variables\n'
        '  3. config-file\n',
        formatter_class=argparse.RawTextHelpFormatter,
        **kwargs,
    ):
        super().__init__(formatter_class=formatter_class, epilog=epilog, **kwargs)
        self.register('action', 'deprecated', DeprecateAction)
        self.add_argument('--auth-config', action='deprecated', help='please use --netdb-config instead')
        self.add_argument('--endpoint', '-e', action='deprecated', help='please use --netdb-enpoint instead')
        self.add_argument('--base-url', '-b', action='deprecated', help='please use --netdb-base-url instead')
        self.add_argument('--token', '-t', action='deprecated', help='please use --netdb-token instead')
        self.add_argument(
            '--netdb-config', default='~/.config/netdb_client.ini', help='config file path (default: %(default)s)'
        )
        self.add_argument(
            '--netdb-endpoint', help='endpoint to use.\n' 'Environment: "NETDB_ENDPOINT"\n' 'Config: [DEFAULT]: endpoint'
        )
        self.add_argument(
            '--netdb-base-url', help='webapi server.\n' 'Environment: "NETDB_BASE_URL"\n' 'Config: [$endpoint]: base_url'
        )
        self.add_argument('--netdb-token', help='user API token.\n' 'Environment: "NETDB_TOKEN"\n' 'Config: [$endpoint]: token')

    def parse_args(self, args=None, namespace=None):
        args = super().parse_args(args, namespace)
        try:
            config = NETDBConfig(Path(args.netdb_config))
        except NETDBConfigError as err:
            self.error(str(err))

        def load_config(option_name, config_key):
            """load config for an option"""

            # if value is already set, we can stop here
            if getattr(args, option_name) is not None:
                return
            envvar_name = f'{option_name.upper()}'
            # try to load option from environment
            value = os.getenv(envvar_name, None)

            if value is None:
                # load value from config section
                value = getattr(config, config_key)(args.netdb_endpoint)

            # For backwards compatibility:
            # if base_url is still unknown use endpoint instead
            if option_name == 'base_url' and value is None:
                value = args.netdb_endpoint

            if value is None:
                self.error(
                    f'No {option_name} specified (looked in args, environment variable '
                    f'"{envvar_name}", config-file in "{args.netdb_config} section {args.netdb_endpoint}")'
                )

            # we have a value now and can use it
            setattr(args, option, value)

        # start with endpoint to get the right config section
        for option, config_key in [('netdb_endpoint', 'endpoint'), ('netdb_base_url', 'base_url'), ('netdb_token', 'token')]:
            load_config(option, config_key)

        return args
