"""click framework helper functions for config loading"""

import types
from pathlib import Path
from typing import Optional
from functools import update_wrapper

import click

from netdb_client.util.config import NETDBConfig


def _get_defaults(config: NETDBConfig, endpoint: str) -> dict[str, str]:
    base_url = config.base_url(endpoint)
    token = config.token(endpoint)

    return {'netdb_base_url': base_url, 'netdb_token': token}


def netdb_config_callback(ctx: click.Context, _param: click.Parameter, config_path: Path):
    if ctx.resilient_parsing:
        return

    config = NETDBConfig(config_path=config_path)
    ctx.obj = config

    default_endpoint = config.default_endpoint
    # if we have no endpoint in config, we can abort here
    if default_endpoint is None:
        return

    ctx.default_map = ctx.default_map or {}  # preserve existing defaults
    ctx.default_map['netdb_endpoint'] = default_endpoint

    ctx.default_map.update(_get_defaults(config, default_endpoint))


def netdb_endpoint_callback(ctx: click.Context, _param: click.Parameter, endpoint: str) -> Optional[str]:
    if ctx.resilient_parsing:
        return

    config = ctx.find_object(NETDBConfig)
    if config is None:
        raise click.ClickException('Could not find config object in context. This should not happen.')
    ctx.default_map = ctx.default_map or {}  # preserve existing defaults

    ctx.default_map.update(_get_defaults(config, endpoint))

    return endpoint


def netdb_config_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--netdb-config', 'netdb_config')

    kwargs.setdefault('callback', netdb_config_callback)
    kwargs.setdefault('is_eager', True)
    kwargs.setdefault('type', click.Path(path_type=Path))
    kwargs.setdefault('default', Path('~/.config/netdb_client.ini'))
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('expose_value', False)
    kwargs.setdefault('help', 'netdb auth config file path')

    return click.option(*param_decls, **kwargs)


def netdb_endpoint_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--netdb-endpoint', 'netdb_endpoint')

    kwargs.setdefault('callback', netdb_endpoint_callback)
    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_ENDPOINT')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('expose_value', False)
    kwargs.setdefault('help', 'endpoint to use. Config: [DEFAULT]: endpoint')

    return click.option(*param_decls, **kwargs)


def netdb_base_url_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--netdb-base-url', 'netdb_base_url')

    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_BASE_URL')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('help', 'webapi server. Config: [$endpoint]: base_url')

    return click.option(*param_decls, **kwargs)


def netdb_token_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--netdb-token', 'netdb_token')

    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_TOKEN')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', False)
    kwargs.setdefault('help', 'user API token. Config: [$endpoint]: token')

    return click.option(*param_decls, **kwargs)


def netdb_api_options(client_lib_module: types.ModuleType, target: str = 'netdb_api_endpoint'):
    def decorator(func):
        @netdb_config_option()
        @netdb_endpoint_option()
        @netdb_base_url_option()
        @netdb_token_option()
        def _decorator(*args, **kwargs):
            api_endpoint = client_lib_module.APIEndpoint(base_url=kwargs.pop('netdb_base_url'), token=kwargs.pop('netdb_token'))
            kwargs[target] = api_endpoint
            return func(*args, **kwargs)

        return update_wrapper(_decorator, func)

    return decorator
